using System;
using System.Text;
using System.IO;
using System.Net;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;
using System.Diagnostics;
using System.Data.SqlTypes;
using System.Reflection;
using System.Threading;
using System.Web;

namespace AXLConnector
{
	/// <summary>
	/// Summary description for CallManagerConnector.
	/// </summary>
	public class CallManagerConnector
	{

		/// <summary>
		/// The CallManagerConnector throws exceptions of type ApplicationException
		/// </summary>
 		public class ApplicationException : Exception
		{
			private string _request;
			private string _response;

			public ApplicationException(string request, string response) {
				_request = request;
				_response = response;
			}

			public string Request {
				get { return _request; }
			}
			public string Response {
				get { return _response; }
			}

			public override string ToString()
			{
				return string.Join(Environment.NewLine, new string[] { base.ToString(), "Callmanager returned an application error:", "Request", _request, "Response", _response } );
			}
		}

		/// <summary>
		/// Constructor to create a connection object. Contains all neccessary data.
		/// </summary>
		/// <param name="host">The host running the service. Can be a dns name or a ip address.</param>
		/// <param name="axlProvisioningService">The releative url of the provisioning service.</param>
		/// <param name="axlServicabilityService">The releative url of the servicability service.</param>
		/// <param name="axlUsername">The username for authentication.</param>
		/// <param name="axlPassword">The password for authentication.</param>
		/// <param name="axlServiceTimeOut">The timeout for soap requests.</param>
		public CallManagerConnector(short id, string host, string axlProvisioningService, string axlServicabilityService, string axlUsername, string axlPassword, int axlServiceTimeOut, string dbName, string dbUserName, string dbPassword)
		{
			ID = id;
			_host = host;
			_axlprovisioningservice = axlProvisioningService;
			_axlservicabilityservice = axlServicabilityService;
			_credential = new NetworkCredential(axlUsername, axlPassword);
			byte [] bytes = Encoding.ASCII.GetBytes(string.Format("{0}:{1}", axlUsername, axlPassword));
			_base64Credentials = Convert.ToBase64String(bytes);
			_axlservicetimeout = axlServiceTimeOut;
		}

		// Default constructor declared private to forbid usage
		private CallManagerConnector()
		{
		}

		private const string soapNs = "http://schemas.xmlsoap.org/soap/envelope";

		// The ccm publisher hosting the service
		private string _host;
		// The relative path to the provisioning service 
		private string _axlprovisioningservice;
		// The relative path to the servicabiilty service 
		private string _axlservicabilityservice;
		// The timeout for requests
		private int _axlservicetimeout;
		// The credentials for the request
		private NetworkCredential _credential;

		private string _base64Credentials = null;

		// The requests encoding
		private Encoding _encoding = Encoding.GetEncoding("ISO-8859-1");
		// The static array of serializers for the different request classes.
		// It's kept static because the production of an XmlSerializer invokes code generation and compilation.
		// The key is the class type of the serialized object.
		private static Hashtable _serializers = new Hashtable();
		private WebProxy _proxy = null;

		public WebProxy Proxy {
			get { return _proxy; }
			set { _proxy = value; }
		}

		public short ID;


		/// <summary>
		/// Get an serializer to serialize/deserialize object of a type
		/// </summary>
		/// <param name="objectType">The classtype to serialize/deserialize</param>
		/// <returns></returns>
		private static XmlSerializer GetSerializer(Type objectType)
		{
			lock (_serializers) {
				if (!_serializers.ContainsKey(objectType)) {
					_serializers.Add(objectType, new XmlSerializer(objectType));

				}
				
                return (XmlSerializer) _serializers[objectType];
			}
		}

		private static XmlSerializer GetSerializer(Type objectType, int usageCount)
		{
			lock (_serializers) {
				if (!_serializers.ContainsKey(objectType)) {
					_serializers.Add(objectType, new XmlSerializer(objectType));
				}

				return (XmlSerializer) _serializers[objectType];
			}
		}


		/// <summary>
		/// Create an AXL Soap Provisioning request. All necessary header data is set.
		/// </summary>
		/// <returns>A HttpWebRequest object.</returns>
		private HttpWebRequest CreateAXLProvisioningRequest()
		{
			HttpWebRequest request = (HttpWebRequest) WebRequest.Create(string.Format("http://{0}{1}", _host, _axlprovisioningservice));
			
			// Don't keep the request alive !!
			request.KeepAlive = false;
			request.ContentType = string.Format("text/xml; charset='{0}'", _encoding.BodyName );

			// Do authentication by hand - otherwise request will always generate 2 requests and the first will get a 401.5 response

			request.Headers.Add("Authorization", "Basic " + _base64Credentials);

			if (_proxy != null) {
				request.Proxy = _proxy;
			}
			request.Accept = "*/*";
			request.Method = "POST";
			request.Timeout = _axlservicetimeout;
			return request;
		}

		/// <summary>
		/// Create an AXL Soap Servicability request. All necessary header data is set.
		/// </summary>
		/// <returns></returns>
		private DeviceQueryService CreateAXLServicabilityRequest()
		{
			DeviceQueryService service = new DeviceQueryService();
			
			service.Url = string.Format("http://{0}{1}", _host, _axlservicabilityservice);
			service.Credentials = _credential;

			// Do preauthentication - otherwise request will always generate 2 requests and the first will get a 401.5 response
			service.PreAuthenticate = true;
			service.AstHeaderValue = new AstHeader();
			service.AstHeaderValue.SessionId = string.Format("{0}-{1:s}", _host, DateTime.Now);
			
			return service;
		}

		/// <summary>
		/// Invoke a provisioning request
		/// </summary>
		/// <param name="axlRequest"></param>
		/// <param name="inputType"></param>
		/// <param name="outputType"></param>
		/// <returns></returns>
		private APIResponse InvokeProvisioningRequest(APIRequest axlRequest, Type inputType, Type outputType)
		{
			XmlSerializer inputSerializer = GetSerializer(inputType);

			HttpWebRequest request = CreateAXLProvisioningRequest();

			XmlTextWriter xw = new XmlTextWriter(request.GetRequestStream(), _encoding);

			xw.WriteStartElement("SOAP-ENV", "Envelope", soapNs);
			xw.WriteStartElement("Body", soapNs);

			inputSerializer.Serialize(xw, axlRequest);
			
			xw.WriteEndElement();
			xw.WriteEndElement();
			xw.Close();
#if DEBUG
			DebugWriteRequest(inputSerializer, axlRequest);
#endif
			HttpWebResponse response = (HttpWebResponse) request.GetResponse();
			StreamReader responseReader = new StreamReader(response.GetResponseStream(), _encoding);
			XmlTextReader xr = new XmlTextReader(responseReader);
			
			XmlDocument doc = new XmlDocument();
			doc.Load(xr);
			xr.Close();

#if DEBUG
			DebugWriteResponse(doc);
#endif

			// Check for errors
			XmlNode faultNode = doc.DocumentElement["SOAP-ENV:Body"]["SOAP-ENV:Fault"];
			if (faultNode != null) {
				throw new CallManagerConnector.ApplicationException(GetFormattedRequest(inputSerializer, axlRequest), GetFormattedResponse(doc));
			} 

			
			string body = doc.DocumentElement["SOAP-ENV:Body"].InnerXml;
			XmlSerializer outputSerializer = GetSerializer(outputType);
			StringReader sr = new StringReader(body);

			APIResponse axlResponse = (APIResponse) outputSerializer.Deserialize(sr);
			return axlResponse;
		}

		private void DebugWriteRequest(XmlSerializer inputSerializer, APIRequest axlRequest)
		{
			Debug.WriteLine("-----------------------------------------------------------");
			Debug.WriteLine(string.Format("Sending AXL Request to {0}", _host));
			Debug.WriteLine("-----------------------------------------------------------");
			
			Debug.WriteLine(GetFormattedRequest(inputSerializer, axlRequest));
			Debug.WriteLine("-----------------------------------------------------------");
		}
		
		private void DebugWriteResponse(XmlDocument responseDoc)
		{
			Debug.WriteLine("-----------------------------------------------------------");
			Debug.WriteLine(string.Format("{0}'s Response", _host));
			Debug.WriteLine("-----------------------------------------------------------");

			Debug.WriteLine(GetFormattedResponse(responseDoc));
			Debug.WriteLine("-----------------------------------------------------------");
		}

		private string GetFormattedRequest(XmlSerializer inputSerializer, APIRequest axlRequest)
		{
			StringBuilder sb = new StringBuilder();
			StringWriter sw = new StringWriter(sb);
			XmlTextWriter dxw = new XmlTextWriter(sw);
			dxw.Formatting = Formatting.Indented;
			dxw.WriteStartElement("SOAP-ENV", "Envelope", soapNs);
			dxw.WriteStartElement("Body", soapNs);
				inputSerializer.Serialize(dxw, axlRequest);
			dxw.WriteEndElement();
			dxw.WriteEndElement();
			dxw.Close();
			return sb.ToString();
		}

		private string GetFormattedResponse(XmlDocument responseDoc)
		{
			StringBuilder sb = new StringBuilder();
			StringWriter sw = new StringWriter(sb);
			XmlTextWriter dxw = new XmlTextWriter(sw);
			dxw.Formatting = Formatting.Indented;
			responseDoc.WriteContentTo(dxw);
			dxw.Close();
			return sb.ToString();
		}

        private Hashtable _stateInfos = new Hashtable();

		// Get the real time status from the devices specified
        // At most 200 devices can be queried at the same time
        // 

        /// <summary>
        ///  Get the real time status from the devices specified. At most 200 devices can be queried at the same time
        /// </summary>
        /// <param name="devicesKey">The devicesKey is used for subsequent queries for the same devices. It must be unique for a list of devices</param>
        /// <param name="devices">An array of CCM device names</param>
        /// <returns>The result of the query</returns>
		public SelectCmDeviceResult GetPhoneStates(string devicesKey, string[] devices)
		{
            if (devices.Length > 200) {
                throw new ArgumentException("At most 200 devices can be queried at the same time");
            }
			DeviceQueryService service = CreateAXLServicabilityRequest();

			CmSelectionCriteria criteria = new CmSelectionCriteria();
			criteria.Class = DeviceClass.Any;
			criteria.Status = CmDevRegStat.Any;
			criteria.MaxReturnedDevices = uint.MaxValue;
            string stateInfo = null;
            
            if (_stateInfos.ContainsKey(devicesKey)) {
                stateInfo = (string) _stateInfos[devicesKey];
            }
            criteria.SelectItems = new SelectItem[devices.Length];

            for (int i = 0; i < devices.Length; i++) {
                criteria.SelectItems[i] = new SelectItem();
                criteria.SelectItems[i].Item = devices[i];
            }

            SelectCmDeviceResult res = service.SelectCmDevice(ref stateInfo, criteria);

            if (!_stateInfos.ContainsKey(devicesKey)) {
                _stateInfos.Add(devicesKey, stateInfo);
            } else {
                _stateInfos[devicesKey] = stateInfo;
            }
            return res;

		}

		public string AxlUUID(object o)
		{
			if (o is Guid || o is SqlGuid) {
				return AxlUUID((Guid) o);
			} else if (o is string) {
				return AxlUUID((string) o);
			} else {
				throw new ArgumentException("Unknown type");
			}
		}

		public string AxlUUID(Guid guid)
		{
			return guid.ToString("B");
		}

		public string AxlUUID(SqlGuid guid)
		{
			return AxlUUID((Guid) guid);
		}

		public string AxlUUID(string guid)
		{
			return AxlUUID(new Guid(guid));
		}

		public string AxlDescription(string description)
		{
			// AXL Servicability Service supplies invalid XML if description contains '&'
			description = description.Replace("&", "");

			if (description.Length > 50) {
				return description.Substring(0, 50);
			} else {
				return description;
			}
		}

		// The generated methods
		public addPhoneResponse addPhone(addPhone input)
		{
			return (addPhoneResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addPhoneResponse));
		}

		public removePhoneResponse removePhone(removePhone input)
		{
			return (removePhoneResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removePhoneResponse));
		}

		public updatePhoneResponse updatePhone(updatePhone input)
		{
			return (updatePhoneResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updatePhoneResponse));
		}

		public getPhoneResponse getPhone(getPhone input)
		{
			return (getPhoneResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getPhoneResponse));
		}

		public doDeviceLoginResponse doDeviceLogin(doDeviceLogin input)
		{
			return (doDeviceLoginResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(doDeviceLoginResponse));
		}

		public doDeviceLogoutResponse doDeviceLogout(doDeviceLogout input)
		{
			return (doDeviceLogoutResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(doDeviceLogoutResponse));
		}

		public doDeviceResetResponse doDeviceReset(doDeviceReset input)
		{
			return (doDeviceResetResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(doDeviceResetResponse));
		}

		public addLineResponse addLine(addLine input)
		{
			return (addLineResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addLineResponse));
		}

		public removeLineResponse removeLine(removeLine input)
		{
			return (removeLineResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeLineResponse));
		}

		public updateLineResponse updateLine(updateLine input)
		{
			return (updateLineResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateLineResponse));
		}

		public getLineResponse getLine(getLine input)
		{
			return (getLineResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getLineResponse));
		}

		public addUserResponse addUser(addUser input)
		{
			return (addUserResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addUserResponse));
		}

		public removeUserResponse removeUser(removeUser input)
		{
			return (removeUserResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeUserResponse));
		}

		public updateUserResponse updateUser(updateUser input)
		{
			return (updateUserResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateUserResponse));
		}

		public getUserResponse getUser(getUser input)
		{
			return (getUserResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getUserResponse));
		}

		public addDeviceProfileResponse addDeviceProfile(addDeviceProfile input)
		{
			return (addDeviceProfileResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addDeviceProfileResponse));
		}

		public removeDeviceProfileResponse removeDeviceProfile(removeDeviceProfile input)
		{
			return (removeDeviceProfileResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeDeviceProfileResponse));
		}

		public updateDeviceProfileResponse updateDeviceProfile(updateDeviceProfile input)
		{
			return (updateDeviceProfileResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateDeviceProfileResponse));
		}

		public getDeviceProfileResponse getDeviceProfile(getDeviceProfile input)
		{
			return (getDeviceProfileResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getDeviceProfileResponse));
		}

		public addRoutePartitionResponse addRoutePartition(addRoutePartition input)
		{
			return (addRoutePartitionResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addRoutePartitionResponse));
		}

		public updateRoutePartitionResponse updateRoutePartition(updateRoutePartition input)
		{
			return (updateRoutePartitionResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateRoutePartitionResponse));
		}

		public removeRoutePartitionResponse removeRoutePartition(removeRoutePartition input)
		{
			return (removeRoutePartitionResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeRoutePartitionResponse));
		}

		public getRoutePartitionResponse getRoutePartition(getRoutePartition input)
		{
			return (getRoutePartitionResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getRoutePartitionResponse));
		}

		public getCSSResponse getCSS(getCSS input)
		{
			return (getCSSResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getCSSResponse));
		}

		public removeCSSResponse removeCSS(removeCSS input)
		{
			return (removeCSSResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeCSSResponse));
		}

		public addCSSResponse addCSS(addCSS input)
		{
			return (addCSSResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addCSSResponse));
		}

		public updateCSSResponse updateCSS(updateCSS input)
		{
			return (updateCSSResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateCSSResponse));
		}

		public getTransPatternResponse getTransPattern(getTransPattern input)
		{
			return (getTransPatternResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getTransPatternResponse));
		}

		public removeTransPatternResponse removeTransPattern(removeTransPattern input)
		{
			return (removeTransPatternResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeTransPatternResponse));
		}

		public addTransPatternResponse addTransPattern(addTransPattern input)
		{
			return (addTransPatternResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addTransPatternResponse));
		}

		public updateTransPatternResponse updateTransPattern(updateTransPattern input)
		{
			return (updateTransPatternResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateTransPatternResponse));
		}

		public getRoutePatternResponse getRoutePattern(getRoutePattern input)
		{
			return (getRoutePatternResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getRoutePatternResponse));
		}

		public removeRoutePatternResponse removeRoutePattern(removeRoutePattern input)
		{
			return (removeRoutePatternResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeRoutePatternResponse));
		}

		public addRoutePatternResponse addRoutePattern(addRoutePattern input)
		{
			return (addRoutePatternResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addRoutePatternResponse));
		}

		public updateRoutePatternResponse updateRoutePattern(updateRoutePattern input)
		{
			return (updateRoutePatternResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateRoutePatternResponse));
		}

		public addRouteGroupResponse addRouteGroup(addRouteGroup input)
		{
			return (addRouteGroupResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addRouteGroupResponse));
		}

		public updateRouteGroupResponse updateRouteGroup(updateRouteGroup input)
		{
			return (updateRouteGroupResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateRouteGroupResponse));
		}

		public getRouteGroupResponse getRouteGroup(getRouteGroup input)
		{
			return (getRouteGroupResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getRouteGroupResponse));
		}

		public removeRouteGroupResponse removeRouteGroup(removeRouteGroup input)
		{
			return (removeRouteGroupResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeRouteGroupResponse));
		}

		public addRouteListResponse addRouteList(addRouteList input)
		{
			return (addRouteListResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addRouteListResponse));
		}

		public updateRouteListResponse updateRouteList(updateRouteList input)
		{
			return (updateRouteListResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateRouteListResponse));
		}

		public getRouteListResponse getRouteList(getRouteList input)
		{
			return (getRouteListResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getRouteListResponse));
		}

		public removeRouteListResponse removeRouteList(removeRouteList input)
		{
			return (removeRouteListResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeRouteListResponse));
		}

		public addCTIRoutePointResponse addCTIRoutePoint(addCTIRoutePoint input)
		{
			return (addCTIRoutePointResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addCTIRoutePointResponse));
		}

		public updateCTIRoutePointResponse updateCTIRoutePoint(updateCTIRoutePoint input)
		{
			return (updateCTIRoutePointResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateCTIRoutePointResponse));
		}

		public getCTIRoutePointResponse getCTIRoutePoint(getCTIRoutePoint input)
		{
			return (getCTIRoutePointResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getCTIRoutePointResponse));
		}

		public removeCTIRoutePointResponse removeCTIRoutePoint(removeCTIRoutePoint input)
		{
			return (removeCTIRoutePointResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeCTIRoutePointResponse));
		}

		public addDialPlanResponse addDialPlan(addDialPlan input)
		{
			return (addDialPlanResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addDialPlanResponse));
		}

		public updateDialPlanResponse updateDialPlan(updateDialPlan input)
		{
			return (updateDialPlanResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateDialPlanResponse));
		}

		public getDialPlanResponse getDialPlan(getDialPlan input)
		{
			return (getDialPlanResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getDialPlanResponse));
		}

		public removeDialPlanResponse removeDialPlan(removeDialPlan input)
		{
			return (removeDialPlanResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeDialPlanResponse));
		}

		public addDialPlanTagResponse addDialPlanTag(addDialPlanTag input)
		{
			return (addDialPlanTagResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addDialPlanTagResponse));
		}

		public updateDialPlanTagResponse updateDialPlanTag(updateDialPlanTag input)
		{
			return (updateDialPlanTagResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateDialPlanTagResponse));
		}

//		public getDialPlanTagResponse getDialPlanTag(getDialPlanTag input)
//		{
//			return (getDialPlanTagResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getDialPlanTagResponse));
//		}

		public removeDialPlanTagResponse removeDialPlanTag(removeDialPlanTag input)
		{
			return (removeDialPlanTagResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeDialPlanTagResponse));
		}

		public addCallParkResponse addCallPark(addCallPark input)
		{
			return (addCallParkResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addCallParkResponse));
		}

		public updateCallParkResponse updateCallPark(updateCallPark input)
		{
			return (updateCallParkResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateCallParkResponse));
		}

		public getCallParkResponse getCallPark(getCallPark input)
		{
			return (getCallParkResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getCallParkResponse));
		}

		public removeCallParkResponse removeCallPark(removeCallPark input)
		{
			return (removeCallParkResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeCallParkResponse));
		}

		public addCallPickupGroupResponse addCallPickupGroup(addCallPickupGroup input)
		{
			return (addCallPickupGroupResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addCallPickupGroupResponse));
		}

		public updateCallPickupGroupResponse updateCallPickupGroup(updateCallPickupGroup input)
		{
			return (updateCallPickupGroupResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateCallPickupGroupResponse));
		}

		public getCallPickupGroupResponse getCallPickupGroup(getCallPickupGroup input)
		{
			return (getCallPickupGroupResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getCallPickupGroupResponse));
		}

		public removeCallPickupGroupResponse removeCallPickupGroup(removeCallPickupGroup input)
		{
			return (removeCallPickupGroupResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeCallPickupGroupResponse));
		}

		public addDDIResponse addDDI(addDDI input)
		{
			return (addDDIResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addDDIResponse));
		}

		public updateDDIResponse updateDDI(updateDDI input)
		{
			return (updateDDIResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateDDIResponse));
		}

//		public getDDIResponse getDDI(getDDI input)
//		{
//			return (getDDIResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getDDIResponse));
//		}

		public removeDDIResponse removeDDI(removeDDI input)
		{
			return (removeDDIResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeDDIResponse));
		}

		public createAutogeneratedProfileResponse createAutogeneratedProfile(createAutogeneratedProfile input)
		{
			return (createAutogeneratedProfileResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(createAutogeneratedProfileResponse));
		}

		public listPhoneByNameResponse listPhoneByName(listPhoneByName input)
		{
			return (listPhoneByNameResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(listPhoneByNameResponse));
		}

		public listPhoneByDescriptionResponse listPhoneByDescription(listPhoneByDescription input)
		{
			return (listPhoneByDescriptionResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(listPhoneByDescriptionResponse));
		}

		public listUserByNameResponse listUserByName(listUserByName input)
		{
			return (listUserByNameResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(listUserByNameResponse));
		}

		public listRoutePlanByTypeResponse listRoutePlanByType(listRoutePlanByType input)
		{
			return (listRoutePlanByTypeResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(listRoutePlanByTypeResponse));
		}

		public getNumDevicesResponse getNumDevices(getNumDevices input)
		{
			return (getNumDevicesResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getNumDevicesResponse));
		}

		public listServiceParametersResponse listServiceParameters(listServiceParameters input)
		{
			return (listServiceParametersResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(listServiceParametersResponse));
		}

		public listCSSByNameResponse listCSSByName(listCSSByName input)
		{
			return (listCSSByNameResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(listCSSByNameResponse));
		}

		public addDevicePoolResponse addDevicePool(addDevicePool input)
		{
			return (addDevicePoolResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addDevicePoolResponse));
		}

		public updateDevicePoolResponse updateDevicePool(updateDevicePool input)
		{
			return (updateDevicePoolResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateDevicePoolResponse));
		}

		public getDevicePoolResponse getDevicePool(getDevicePool input)
		{
			return (getDevicePoolResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getDevicePoolResponse));
		}

		public removeDevicePoolResponse removeDevicePool(removeDevicePool input)
		{
			return (removeDevicePoolResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeDevicePoolResponse));
		}

		public addCallManagerGroupResponse addCallManagerGroup(addCallManagerGroup input)
		{
			return (addCallManagerGroupResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addCallManagerGroupResponse));
		}

		public updateCallManagerGroupResponse updateCallManagerGroup(updateCallManagerGroup input)
		{
			return (updateCallManagerGroupResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateCallManagerGroupResponse));
		}

		public getCallManagerGroupResponse getCallManagerGroup(getCallManagerGroup input)
		{
			return (getCallManagerGroupResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getCallManagerGroupResponse));
		}

		public removeCallManagerGroupResponse removeCallManagerGroup(removeCallManagerGroup input)
		{
			return (removeCallManagerGroupResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeCallManagerGroupResponse));
		}

		public addVoiceMailPortResponse addVoiceMailPort(addVoiceMailPort input)
		{
			return (addVoiceMailPortResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addVoiceMailPortResponse));
		}

		public updateVoiceMailPortResponse updateVoiceMailPort(updateVoiceMailPort input)
		{
			return (updateVoiceMailPortResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateVoiceMailPortResponse));
		}

		public getVoiceMailPortResponse getVoiceMailPort(getVoiceMailPort input)
		{
			return (getVoiceMailPortResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getVoiceMailPortResponse));
		}

		public removeVoiceMailPortResponse removeVoiceMailPort(removeVoiceMailPort input)
		{
			return (removeVoiceMailPortResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeVoiceMailPortResponse));
		}

		public addGatewayEndpointResponse addGatewayEndpoint(addGatewayEndpoint input)
		{
			return (addGatewayEndpointResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addGatewayEndpointResponse));
		}

		public updateGatewayEndpointResponse updateGatewayEndpoint(updateGatewayEndpoint input)
		{
			return (updateGatewayEndpointResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateGatewayEndpointResponse));
		}

		public getGatewayEndpointResponse getGatewayEndpoint(getGatewayEndpoint input)
		{
			return (getGatewayEndpointResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getGatewayEndpointResponse));
		}

		public removeGatewayEndpointResponse removeGatewayEndpoint(removeGatewayEndpoint input)
		{
			return (removeGatewayEndpointResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeGatewayEndpointResponse));
		}

		public addRouteFilterResponse addRouteFilter(addRouteFilter input)
		{
			return (addRouteFilterResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addRouteFilterResponse));
		}

		public updateRouteFilterResponse updateRouteFilter(updateRouteFilter input)
		{
			return (updateRouteFilterResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateRouteFilterResponse));
		}

		public getRouteFilterResponse getRouteFilter(getRouteFilter input)
		{
			return (getRouteFilterResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getRouteFilterResponse));
		}

		public removeRouteFilterResponse removeRouteFilter(removeRouteFilter input)
		{
			return (removeRouteFilterResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeRouteFilterResponse));
		}

		public getServiceParameterResponse getServiceParameter(getServiceParameter input)
		{
			return (getServiceParameterResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getServiceParameterResponse));
		}

		public updateServiceParameterResponse updateServiceParameter(updateServiceParameter input)
		{
			return (updateServiceParameterResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateServiceParameterResponse));
		}

		public addLocationResponse addLocation(addLocation input)
		{
			return (addLocationResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addLocationResponse));
		}

		public updateLocationResponse updateLocation(updateLocation input)
		{
			return (updateLocationResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateLocationResponse));
		}

		public getLocationResponse getLocation(getLocation input)
		{
			return (getLocationResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getLocationResponse));
		}

		public removeLocationResponse removeLocation(removeLocation input)
		{
			return (removeLocationResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeLocationResponse));
		}

		public addRegionResponse addRegion(addRegion input)
		{
			return (addRegionResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addRegionResponse));
		}

		public removeRegionResponse removeRegion(removeRegion input)
		{
			return (removeRegionResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeRegionResponse));
		}

		public getRegionResponse getRegion(getRegion input)
		{
			return (getRegionResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getRegionResponse));
		}

		public updateRegionMatrixResponse updateRegionMatrix(updateRegionMatrix input)
		{
			return (updateRegionMatrixResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateRegionMatrixResponse));
		}

		public addMGCPResponse addMGCP(addMGCP input)
		{
			return (addMGCPResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addMGCPResponse));
		}

		public addMGCPUnitResponse addMGCPUnit(addMGCPUnit input)
		{
			return (addMGCPUnitResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addMGCPUnitResponse));
		}

		public addMGCPSubunitResponse addMGCPSubunit(addMGCPSubunit input)
		{
			return (addMGCPSubunitResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addMGCPSubunitResponse));
		}

		public addMGCPEndpointResponse addMGCPEndpoint(addMGCPEndpoint input)
		{
			return (addMGCPEndpointResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addMGCPEndpointResponse));
		}

		public getMGCPResponse getMGCP(getMGCP input)
		{
			return (getMGCPResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getMGCPResponse));
		}

		public removeMGCPResponse removeMGCP(removeMGCP input)
		{
			return (removeMGCPResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeMGCPResponse));
		}

		public removeMGCPUnitResponse removeMGCPUnit(removeMGCPUnit input)
		{
			return (removeMGCPUnitResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeMGCPUnitResponse));
		}

		public removeMGCPSubunitResponse removeMGCPSubunit(removeMGCPSubunit input)
		{
			return (removeMGCPSubunitResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeMGCPSubunitResponse));
		}

		public removeMGCPEndpointResponse removeMGCPEndpoint(removeMGCPEndpoint input)
		{
			return (removeMGCPEndpointResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeMGCPEndpointResponse));
		}

		public updateMGCPResponse updateMGCP(updateMGCP input)
		{
			return (updateMGCPResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateMGCPResponse));
		}

		public addCallManagerResponse addCallManager(addCallManager input)
		{
			return (addCallManagerResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addCallManagerResponse));
		}

		public updateCallManagerResponse updateCallManager(updateCallManager input)
		{
			return (updateCallManagerResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateCallManagerResponse));
		}

		public getCallManagerResponse getCallManager(getCallManager input)
		{
			return (getCallManagerResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getCallManagerResponse));
		}

		public removeCallManagerResponse removeCallManager(removeCallManager input)
		{
			return (removeCallManagerResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeCallManagerResponse));
		}

		public addProcessNodeResponse addProcessNode(addProcessNode input)
		{
			return (addProcessNodeResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addProcessNodeResponse));
		}

		public updateProcessNodeResponse updateProcessNode(updateProcessNode input)
		{
			return (updateProcessNodeResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateProcessNodeResponse));
		}

		public getProcessNodeResponse getProcessNode(getProcessNode input)
		{
			return (getProcessNodeResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getProcessNodeResponse));
		}

		public removeProcessNodeResponse removeProcessNode(removeProcessNode input)
		{
			return (removeProcessNodeResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeProcessNodeResponse));
		}

		public listAllProcessNodesResponse listAllProcessNodes(listAllProcessNodes input)
		{
			return (listAllProcessNodesResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(listAllProcessNodesResponse));
		}

		public listProcessNodesByServiceResponse listProcessNodesByService(listProcessNodesByService input)
		{
			return (listProcessNodesByServiceResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(listProcessNodesByServiceResponse));
		}

		public getProcessNodeServiceResponse getProcessNodeService(getProcessNodeService input)
		{
			return (getProcessNodeServiceResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getProcessNodeServiceResponse));
		}

		public updateProcessNodeServiceResponse updateProcessNodeService(updateProcessNodeService input)
		{
			return (updateProcessNodeServiceResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateProcessNodeServiceResponse));
		}

		public listRoutePartitionByNameResponse listRoutePartitionByName(listRoutePartitionByName input)
		{
			return (listRoutePartitionByNameResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(listRoutePartitionByNameResponse));
		}

		public listDevicePoolByNameResponse listDevicePoolByName(listDevicePoolByName input)
		{
			return (listDevicePoolByNameResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(listDevicePoolByNameResponse));
		}

//		public listPhoneTemplateByNameResponse listPhoneTemplateByName(listPhoneTemplateByName input)
//		{
//			return (listPhoneTemplateByNameResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(listPhoneTemplateByNameResponse));
//		}

		public listDeviceByNameAndClassResponse listDeviceByNameAndClass(listDeviceByNameAndClass input)
		{
			return (listDeviceByNameAndClassResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(listDeviceByNameAndClassResponse));
		}

		public listMOHAudioSourceByNameResponse listMOHAudioSourceByName(listMOHAudioSourceByName input)
		{
			return (listMOHAudioSourceByNameResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(listMOHAudioSourceByNameResponse));
		}

		public updateMOHAudioSourceResponse updateMOHAudioSource(updateMOHAudioSource input)
		{
			return (updateMOHAudioSourceResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateMOHAudioSourceResponse));
		}

		public getMOHAudioSourceResponse getMOHAudioSource(getMOHAudioSource input)
		{
			return (getMOHAudioSourceResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getMOHAudioSourceResponse));
		}

		public removeMOHAudioSourceResponse removeMOHAudioSource(removeMOHAudioSource input)
		{
			return (removeMOHAudioSourceResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeMOHAudioSourceResponse));
		}

		public listMediaResourceListByNameResponse listMediaResourceListByName(listMediaResourceListByName input)
		{
			return (listMediaResourceListByNameResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(listMediaResourceListByNameResponse));
		}

		public addMediaResourceListResponse addMediaResourceList(addMediaResourceList input)
		{
			return (addMediaResourceListResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addMediaResourceListResponse));
		}

		public getMediaResourceListResponse getMediaResourceList(getMediaResourceList input)
		{
			return (getMediaResourceListResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getMediaResourceListResponse));
		}

		public removeMediaResourceListResponse removeMediaResourceList(removeMediaResourceList input)
		{
			return (removeMediaResourceListResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeMediaResourceListResponse));
		}

		public updateMediaResourceListResponse updateMediaResourceList(updateMediaResourceList input)
		{
			return (updateMediaResourceListResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateMediaResourceListResponse));
		}

		public listMediaResourceGroupByNameResponse listMediaResourceGroupByName(listMediaResourceGroupByName input)
		{
			return (listMediaResourceGroupByNameResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(listMediaResourceGroupByNameResponse));
		}

		public addMediaResourceGroupResponse addMediaResourceGroup(addMediaResourceGroup input)
		{
			return (addMediaResourceGroupResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addMediaResourceGroupResponse));
		}

		public getMediaResourceGroupResponse getMediaResourceGroup(getMediaResourceGroup input)
		{
			return (getMediaResourceGroupResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getMediaResourceGroupResponse));
		}

		public removeMediaResourceGroupResponse removeMediaResourceGroup(removeMediaResourceGroup input)
		{
			return (removeMediaResourceGroupResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeMediaResourceGroupResponse));
		}

		public updateMediaResourceGroupResponse updateMediaResourceGroup(updateMediaResourceGroup input)
		{
			return (updateMediaResourceGroupResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateMediaResourceGroupResponse));
		}

		public listLocationByNameResponse listLocationByName(listLocationByName input)
		{
			return (listLocationByNameResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(listLocationByNameResponse));
		}

		public listAARGroupByNameResponse listAARGroupByName(listAARGroupByName input)
		{
			return (listAARGroupByNameResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(listAARGroupByNameResponse));
		}

		public listVoiceMailProfileByNameResponse listVoiceMailProfileByName(listVoiceMailProfileByName input)
		{
			return (listVoiceMailProfileByNameResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(listVoiceMailProfileByNameResponse));
		}

		public listGatekeeperByNameResponse listGatekeeperByName(listGatekeeperByName input)
		{
			return (listGatekeeperByNameResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(listGatekeeperByNameResponse));
		}

		public addGatekeeperResponse addGatekeeper(addGatekeeper input)
		{
			return (addGatekeeperResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addGatekeeperResponse));
		}

		public getGatekeeperResponse getGatekeeper(getGatekeeper input)
		{
			return (getGatekeeperResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getGatekeeperResponse));
		}

		public removeGatekeeperResponse removeGatekeeper(removeGatekeeper input)
		{
			return (removeGatekeeperResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeGatekeeperResponse));
		}

		public updateGatekeeperResponse updateGatekeeper(updateGatekeeper input)
		{
			return (updateGatekeeperResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateGatekeeperResponse));
		}

		public addH323PhoneResponse addH323Phone(addH323Phone input)
		{
			return (addH323PhoneResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addH323PhoneResponse));
		}

		public addH323GatewayResponse addH323Gateway(addH323Gateway input)
		{
			return (addH323GatewayResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addH323GatewayResponse));
		}

		public addH323TrunkResponse addH323Trunk(addH323Trunk input)
		{
			return (addH323TrunkResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addH323TrunkResponse));
		}

		public getH323PhoneResponse getH323Phone(getH323Phone input)
		{
			return (getH323PhoneResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getH323PhoneResponse));
		}

		public getH323GatewayResponse getH323Gateway(getH323Gateway input)
		{
			return (getH323GatewayResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getH323GatewayResponse));
		}

		public getH323TrunkResponse getH323Trunk(getH323Trunk input)
		{
			return (getH323TrunkResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getH323TrunkResponse));
		}

		public removeH323PhoneResponse removeH323Phone(removeH323Phone input)
		{
			return (removeH323PhoneResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeH323PhoneResponse));
		}

		public removeH323GatewayResponse removeH323Gateway(removeH323Gateway input)
		{
			return (removeH323GatewayResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeH323GatewayResponse));
		}

		public removeH323TrunkResponse removeH323Trunk(removeH323Trunk input)
		{
			return (removeH323TrunkResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeH323TrunkResponse));
		}

		public updateH323PhoneResponse updateH323Phone(updateH323Phone input)
		{
			return (updateH323PhoneResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateH323PhoneResponse));
		}

		public updateH323GatewayResponse updateH323Gateway(updateH323Gateway input)
		{
			return (updateH323GatewayResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateH323GatewayResponse));
		}

		public updateH323TrunkResponse updateH323Trunk(updateH323Trunk input)
		{
			return (updateH323TrunkResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateH323TrunkResponse));
		}

		public executeSQLQueryResponse executeSQLQuery(executeSQLQuery input)
		{
			return (executeSQLQueryResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(executeSQLQueryResponse));
		}

		public addLineGroupResponse addLineGroup(addLineGroup input)
		{
			return (addLineGroupResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addLineGroupResponse));
		}

		public getLineGroupResponse getLineGroup(getLineGroup input)
		{
			return (getLineGroupResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getLineGroupResponse));
		}

		public removeLineGroupResponse removeLineGroup(removeLineGroup input)
		{
			return (removeLineGroupResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeLineGroupResponse));
		}

		public updateLineGroupResponse updateLineGroup(updateLineGroup input)
		{
			return (updateLineGroupResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateLineGroupResponse));
		}

		public addHuntListResponse addHuntList(addHuntList input)
		{
			return (addHuntListResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addHuntListResponse));
		}

		/*public getHuntListResponse getHuntList(getHuntList input)
		{
			return (getHuntListResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getHuntListResponse));
		}*/

		public removeHuntListResponse removeHuntList(removeHuntList input)
		{
			return (removeHuntListResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeHuntListResponse));
		}

		public updateHuntListResponse updateHuntList(updateHuntList input)
		{
			return (updateHuntListResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateHuntListResponse));
		}

		public addHuntPilotResponse addHuntPilot(addHuntPilot input)
		{
			return (addHuntPilotResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(addHuntPilotResponse));
		}

		public getHuntPilotResponse getHuntPilot(getHuntPilot input)
		{
			return (getHuntPilotResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(getHuntPilotResponse));
		}

		public removeHuntPilotResponse removeHuntPilot(removeHuntPilot input)
		{
			return (removeHuntPilotResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(removeHuntPilotResponse));
		}

		public updateHuntPilotResponse updateHuntPilot(updateHuntPilot input)
		{
			return (updateHuntPilotResponse) InvokeProvisioningRequest(input, input.GetType(), typeof(updateHuntPilotResponse));
		}


        public override string ToString()
        {
            return string.Format("Host {0}", _host);
        }
	}
}
